/*
 *
 * wotan.snmp.c v0.23
 * (c)1999 Marek Uher, Marek.Uher@uhkt.cz
 *
 * Description:
 * Simple SNMP query tool - communicates with remote host using SNMP protocol
 *
 */

/*
 * 
 *
 *			   Copyright 1999 by Marek Uher
 *
 *				All Rights Reserved
 *
 * Permission  to  use,  copy,  modify,  and  distribute  this  software and its
 * documentation  for  any  purpose  and without fee is hereby granted. Provided
 * that  that  the  name  Marek Uher  not  be  used  in advertising or publicity
 * pertaining  to  distribution  of the software without specific, written prior
 * permission.
 *
 * Marek Uher DISCLAIMS  ALL  WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED  WARRANTIES  OF  MERCHANTABILITY  AND  FITNESS, IN NO EVENT SHALL
 * Marek Uher BE LIABLE  FOR  ANY  SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER  RESULTING  FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *
 */

/*
 *	snmpwalk.c
 *
 *	Copyright 1988, 1989 by Carnegie Mellon University
 *
 *	All Rights Reserved
 *
 *	Permission to use, copy, modify, and distribute this software and its
 *	documentation for any purpose and without fee is hereby granted,
 *	provided that the above copyright notice appear in all copies and that
 *	both that copyright notice and this permission notice appear in
 *	supporting documentation, and that the name of CMU not be
 *	used in advertising or publicity pertaining to distribution of the
 *	software without specific, written prior permission.
 *
 *	CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 *	ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 *	CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 *	ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 *	WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 *	ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 *	SOFTWARE.
 *
 */

/*
 *	System header files (directory /usr/include)
 */
#include <sys/types.h>
#include <netinet/in.h>
#include <stdio.h>

/*
 *	Ident informations
 */
static char const rcsid[]	= "$Id: wotan.snmp.c,v 0.23 1999/11/01 16:14:07 quark Exp $";
static char const author[]	= "$Author: Marek Uher, Marek.Uher@uhkt.cz $";
static char const platform[]	= "$Platform: Linux 2.0.x $";

/*
 *	Header files specific for Linux
 */
#ifdef Linux
	#include <stdlib.h>
	#include <string.h>
	#include <unistd.h>
#endif

/*
 *	Header file for manipulating with MIB from CMU SNMP
 */
#include "mib.h"

/*
 *	Header files for SNMP (directory /usr/include/snmp)
 */
#include "snmp.h"
#include "snmp_impl.h"
#include "asn1.h"
#include "snmp_api.h"
#include "snmp_client.h"

/*
 *	Definition of constatnts
 */
#define OK	0
#define NONE	0
#define EMPTY	0
#define FULL	1
#define NO	0
#define YES	1
#define CORECTION


/*
 *	Global variables for manipulating with SNMP
 */
 
/*
 *	MIB object indentifier
 */
oid MIBView[] = { 1, 3, 6, 1, 2, 1 };

/*
 *	Dumping packet flag
 */
int SNMPDumpingPacketFlag = NO;

/*
 *	Get help messages
 */
static void GetHelp (void)
{
    fprintf (stderr, "Usage: wotan.snmp [options] host community [object identifier]\n");
    fprintf (stderr, "Valid options are:\n");
    fprintf (stderr, " -p <port>    - SNMP listening port (default port: 161)\n");
    fprintf (stderr, " -T <timeout> - initial timeout in ms (default timeout: %d ms)\n",
	     (int) (SNMP_API_DEFAULT_TIMEOUT / 1000));
    fprintf (stderr, " -R <retries> - number of retries (default: %d retries)\n",
	     SNMP_API_DEFAULT_RETRIES);
    exit (1);
}

/*
 *	Main function
 */
int main (int argc, char *argv[])
{
	/*
	 *	Variables for manipulating with SNMP objects
	 */
	struct snmp_session mySession, *mySNMPSession;
	struct snmp_pdu *pdu, *incomingAnswer = NONE;
	struct variable_list *mySNMPVariableList;
	int argument;
	
	/*
	 *	Variables which contain information about remote host
	 */ 
	char *remoteHost		= NULL;
	char *remoteHostCommunity	= NULL;
	int remoteHostMIBRoot		= EMPTY;
	oid SNMPObjectName[32];
	int SNMPObjectNameSize		= NONE;
	oid MIBObjectIdentifier[MAX_NAME_LEN];
	int MIBObjectSize		= NONE;
	
	/*
	 *	Miscellaneous variables
	 */
	int counter		= 0;
	int runTimeFlag		= NO;
	int programState	= NONE;
	int portFlag		= NO;
	int destinationFlag	= NO;
	int timeOutFlag		= NO;
	int timeOutValue	= NONE;
	int repeatFlag		= NO;
	int repeatValue		= NONE;

	/*
	 *	Init MIB
	 */
	init_mib ();

	/*
	 * 	Parse arguments from command line
	 *	Syntax: wotan.snmp host community [object identifier]
	 */
	for(argument = 1; argument < argc; argument++) {
		if (argv[argument][0] == '-') {
			switch (argv[argument][1]) {

				case 'd':
					SNMPDumpingPacketFlag++;
					break;

				case 'p':
					portFlag = YES;
					if (argument + 1 >= argc) {
						GetHelp ();
					}
					destinationFlag = atoi(argv[++argument]);
					break;

				case 'T':
					timeOutFlag = YES;
					if (argument + 1 >= argc) {
						GetHelp ();
					}
					timeOutValue = atoi(argv[++argument]);
					break;

				case 'R':
					repeatFlag = YES;
					if (argument + 1 >= argc) {
						GetHelp ();
					}
					repeatValue = atoi(argv[++argument]);
					break;

				default:
					printf("Unknown option: -%c\n", argv[argument][1]);
					GetHelp ();
					break;
			}
			continue;
		}
	
		if (remoteHost == NULL) {
			remoteHost = argv[argument];
		} 
		else {
			if (remoteHostCommunity == NULL) {
				remoteHostCommunity = argv[argument]; 
			} 
			else {
				MIBObjectSize = MAX_NAME_LEN;
				if (read_objid(argv[argument], MIBObjectIdentifier, &MIBObjectSize)) {
					remoteHostMIBRoot = FULL;
				}
				else {
					printf("Invalid object identifier: %s\n", argv[argument]);
				}
			}
		}
	}

	if (remoteHostMIBRoot == EMPTY) {
		bcopy((char *)MIBView, (char *)MIBObjectIdentifier, sizeof(MIBView));
		MIBObjectSize = sizeof(MIBView) / sizeof(oid);
		remoteHostMIBRoot = FULL;
	}

	if (!(remoteHost && remoteHostCommunity && remoteHostMIBRoot == 1)) {
		printf("Usage: wotan.snmp [options] host community [object identifier]\n");
		exit(1);
	}

	bzero((char *)&mySession, sizeof(struct snmp_session));
	mySession.peername = remoteHost;
	
	if (portFlag) {
		mySession.remote_port = destinationFlag;
	}

	mySession.community = (u_char *)remoteHostCommunity;
	mySession.community_len = strlen((char *)remoteHostCommunity);

	if (repeatFlag) {
		mySession.retries = repeatValue;
	}
	else {
		mySession.retries = SNMP_DEFAULT_RETRIES;
	}
	
	if (timeOutFlag) {
		mySession.timeout = timeOutValue * 1024;
	}
	else {
		mySession.timeout = SNMP_DEFAULT_TIMEOUT;
	}
	
	mySession.authenticator = NULL;
	snmp_synch_setup(&mySession);
	mySNMPSession = snmp_open(&mySession);

	if (mySNMPSession == NULL) {
		printf("Nelze otevrit snmp\n");
		exit(-1);
	}

	bcopy((char *)MIBObjectIdentifier, (char *)SNMPObjectName, MIBObjectSize * sizeof(oid));
	SNMPObjectNameSize = MIBObjectSize;

	runTimeFlag = YES;

	while(runTimeFlag) {
	
		runTimeFlag = NO;
		pdu = snmp_pdu_create(GETNEXT_REQ_MSG);
		snmp_add_null_var(pdu, SNMPObjectName, SNMPObjectNameSize);
		programState = snmp_synch_response(mySNMPSession, pdu, &incomingAnswer);

		if (programState == STAT_SUCCESS) {
		
			if (incomingAnswer->errstat == SNMP_ERR_NOERROR) {
		
				if (incomingAnswer->command == REPORT_MSG) {
			
					printf ("This is an error report.\n");
					for(mySNMPVariableList = incomingAnswer->variables; mySNMPVariableList; 
					    mySNMPVariableList = mySNMPVariableList->next_variable) {
						print_variable(mySNMPVariableList->name, mySNMPVariableList->name_length, mySNMPVariableList);
					}
				}

				for(mySNMPVariableList = incomingAnswer->variables; mySNMPVariableList; 
				    mySNMPVariableList = mySNMPVariableList->next_variable) {
					if (mySNMPVariableList->name_length < MIBObjectSize || bcmp(MIBObjectIdentifier, mySNMPVariableList->name, MIBObjectSize * sizeof(oid))) {
						continue;
					}
	
					if( mySNMPVariableList->type != SNMP_ENDOFMIBVIEW ) {
						if( incomingAnswer->command == REPORT_MSG ) {
							printf( "This is an error report\n" );
						}
						print_variable(mySNMPVariableList->name, mySNMPVariableList->name_length, mySNMPVariableList);
						bcopy((char *)mySNMPVariableList->name, (char *)SNMPObjectName, mySNMPVariableList->name_length * sizeof(oid));
						SNMPObjectNameSize = mySNMPVariableList->name_length;
						if( incomingAnswer->command != REPORT_MSG ) {
							runTimeFlag = YES; 
						}
					}
				}
			}
			else {
				if (incomingAnswer->errstat == SNMP_ERR_NOSUCHNAME) {
					printf("End of MIB.\n");
				}
				else {
					printf("Error in received packet.\nReason: %s\n", snmp_errstring(incomingAnswer->errstat));
					if (incomingAnswer->errstat == SNMP_ERR_NOSUCHNAME) {
	
						printf("Object identifier request faild for this objects: ");
	
						for(counter = 1, mySNMPVariableList = incomingAnswer->variables; mySNMPVariableList && counter != incomingAnswer->errindex;
						    mySNMPVariableList = mySNMPVariableList->next_variable, counter++)
							CORECTION;
						if (mySNMPVariableList) {
							print_objid(mySNMPVariableList->name, mySNMPVariableList->name_length);
						}
						printf("\n");
					}
				}
			}
		}
		else {
			if (programState == STAT_TIMEOUT) {
				printf("Remote system %s not responding\n", remoteHost);
				exit (1);
			}
			else {    
				printf("Error! Exiting...\n");
				exit (1);
			}
		}
		
		if (incomingAnswer) {
			snmp_free_pdu(incomingAnswer);
			incomingAnswer = NONE;
		}
	}
	
	snmp_close(mySNMPSession);
	return OK;
}

/* End of file wotan.snmp.c */
